import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.zip.GZIPInputStream;

/**
 * Represents a map file, this is used due to no mapdata released for revision
 * #498, we are going to load 474 maps, and use them instead.
 * 
 * @author Thomas Le Godais
 * @author Zee Beast
 */
public class MapFile {

	/**
	 * A {@link ArrayList} for storing the map indexes.
	 */
	public static ArrayList<Integer> mapIndex;
	
	/**
	 * A {@link Map} for storing the maps.
 	 */
	public static Map<Integer, byte[]> mapContainer = new HashMap<Integer, byte[]>();
	
	/**
	 * The directory for storing maps.
	 */
	public static final String DIRECTORY = "./cache/maps/";

	/**
	 * Gets the map by the id.
	 * 
	 * @param mapId The id of the map.
	 * @return The map.
	 * @throws IOException 
	 */
	public static byte[] getMap(int mapId) throws IOException {
		
		// Load the index, if needed.
		if(mapIndex == null) {
			mapIndex = new ArrayList<Integer>();
			DataInputStream dataInput = new DataInputStream(new FileInputStream(DIRECTORY + "mapIndex.idx2"));
			long fileLength = new File(DIRECTORY + "mapIndex.idx2").length() / 2;
			for(int i = 0; i < fileLength; i++) {
				mapIndex.add((int) dataInput.readShort());
			}
		} 
		
		// Check to see if the file exists.
		File file = new File(DIRECTORY + mapId + ".dat");
		if(file.exists()) {
			byte[] b = readMap(DIRECTORY + mapId + ".dat");
			return b;
		} else if(mapContainer.get(mapId) == null) {
			RandomAccessFile dataFile = new RandomAccessFile(DIRECTORY + "maps.dat", "rw");
			RandomAccessFile indexFile = new RandomAccessFile(DIRECTORY + "mapIndex.idx", "rw");
			
			int mapPosition = getMapPosition(mapId);
			if(mapPosition == -1) {
				return null;
			}
			
			indexFile.seek(mapPosition * 12);
			dataFile.seek(indexFile.readInt());
			
			byte[] b = new byte[indexFile.readInt()];
			dataFile.readFully(b);
			b = inflateMap(b, indexFile.readInt());
			
			mapContainer.put(mapId, b);
		}
		return mapContainer.get(mapId);
	}

	/**
	 * Inflates a map.
	 * 
	 * @param b The byte array.
	 * @param length The length.
	 * @return 
	 */
	private static byte[] inflateMap(byte[] b, int length) throws IOException {
		byte[] buffer = new byte[length];
		
		ByteArrayInputStream byteInput = new ByteArrayInputStream(b);
		DataInputStream dataInput = new DataInputStream(new GZIPInputStream(byteInput));
		
		dataInput.readFully(buffer, 0, buffer.length);
		dataInput.close();
		return buffer;
	}

	/**
	 * Gets the position of a map, by the ID.
	 * 
	 * @param mapId The map id.
	 * @return The position.
	 */
	private static int getMapPosition(int mapId) throws IOException {
		if(mapIndex.contains(mapId)) {
			for(int i = 0; i < mapIndex.size(); i++) {
				if(mapIndex.get(i) == mapId) {
					return i;
				}
			}
		}
		return -1;
	}

	/**
	 * Reads a map file.
	 * 
	 * @param directory The directory.
	 * @return The read map.
	 * @throws IOException An error reading.
 	 */
	public static byte[] readMap(String directory) throws IOException {
		File file = new File(directory);
		byte[] storeLength = new byte[(int) file.length()];
		
		DataInputStream dataInput = new DataInputStream(new FileInputStream(directory));
		dataInput.readFully(storeLength, 0, (int) file.length());
		dataInput.close();
		
		return storeLength;
	}
}
